#!/usr/bin/env python

# DO GLOBAL IMPORTS
import matplotlib.dates as mdates
import rrdtool
import sys , utils
import numpy    as np
import numpy.ma as ma

SECONDS_IN_DAY  = 86400
SECONDS_IN_WEEK = 604800
SECONDS_IN_MONTH= 2629743
SECONDS_IN_YEAR = 31556926

def info( rrd_info , options , xmld ):
    
    import matplotlib
    import itertools
    color = itertools.cycle([ 'red' , 'blue' , 'green' , 'gray' ])
    
    if options.graph:
        matplotlib.use('Agg')
        matplotlib.rc('xtick',labelsize = 8)
        matplotlib.rc('ytick',labelsize = 8)
        matplotlib.rc('legend',fontsize = 8)
        import matplotlib.pyplot as plt
        
        #~ SET SIZE OF GRAPH
        size = map(int,options.size.split('x'))
        fig  = plt.figure(figsize = size)
        ax   = plt.subplot(111)
        # Define our color iterator that will be used during graphing
    
    if options.xml:
        #~ from BeautifulSoup import BeautifulSoup as soup, Tag as Tag, NavigableString as nav
        if options.dates:
            dates = options.dates.split(',')
        else:
            dates = ''
        #~ XML     = soup()
        #~ root    = Tag(XML,"answers")
        #~ XML.insert(0,root)
        #~ for i,date in enumerate(dates):
            #~ vars()['answer'+str(i)] = Tag(XML,"answer")
            #~ vars()['answer'+str(i)]['id'] = date
        values  = []
        metrics = []
            
    
    def add_to_graph( contiguous , xmutable , yset , plot_color , s_multiplier = lambda x: x, e_multiplier = lambda x: x , title = '' , units = '' , trans = .15):
        
        #~ print xmutable, yset
        #~ Just plot one to get the title. Otherwise if there are discontinuities
        #~ there will be lots of labels.
        if units:
            title = title + ' (' + units + ')'
        ax.plot(xmutable[0],yset[0],color = plot_color , label = title)
        for slices in contiguous:
            # If smoothing happened, modify the slice count
            # Slice xpoints and yset and assign them to x,y
            mod_slice       = slice(s_multiplier(slices.start),
                                    e_multiplier(slices.stop))
            #~ print "Mod start",mod_slice.start
            #~ print "Mod stop",mod_slice.stop
            x = xmutable[mod_slice]
            y = yset[mod_slice]
            ax.plot(x,y,color = plot_color)
            ax.fill_between(x,y,color = plot_color,alpha = trans)
    #~ REASSIGN PARSED ARG VALUES HERE FOR CLARITY
    smoothing_is_wanted = options.smooth    # Will normally be options.smooth
    #~ REMOVED AND FUNCTIONALITY MOVED TO getRRDinfo
    #~ index  = 0                              # Will normally be options.index
    #~ TODO PUT RRD CODE HERE
    #~ FOR NOW WE ASSUME THAT THE RRD WILL MAKE LIST OF YPOINTS NAMED
    #~ ypoints, resolution and start and end dates
    #~ TEST VALUES:
    resolution  = rrd_info['res']
    start       = rrd_info['start']
    end         = rrd_info['end']
    ypoints     = rrd_info['ypoints']
    extrapolate = int(options.extrapolate)
    method      = options.method
    #~ period      = 
    #~ Make x-axis  points out of how many points are in a ypoints list
    xpoints     = matplotlib.dates.epoch2num(ma.array(range(start,end,resolution)))
    #~ If we are extralating points, determine the x points in the extrapolation
    if extrapolate:
        from utils import lengthOfPeriod
        extra    = lengthOfPeriod(options.period) * extrapolate
        cycle    = int(lengthOfPeriod(options.period) / resolution)
        xlargest = matplotlib.dates.epoch2num(ma.array(range(start,end + extra,resolution)))
        xextrap  = matplotlib.dates.epoch2num(ma.array(range(end,end + extra,resolution)))
    else:
        xlargest = ma.copy(xpoints)
    #~ Now its time to graph the points:
    for i,yset in enumerate(ypoints):
        #~ Try adding our label
        if xmld.get('title',False):
            title   = xmld['title'][i]
        else:
            title   = ''
        if xmld.get('units',False):
            #~ Test if list contains all one item
            if xmld['units'][1:] == xmld['units'][:-1]:
                unit = ''
            else:
                unit    = xmld['units'][i]
        else:
            unit    = ''
        # Determine our ymask
        ymasked     = ma.masked_equal(yset,None)
        ymask       = ymasked.mask
        # Get contigous not masked data
        contiguous  = ma.flatnotmasked_contiguous(ymasked)
        # Define new paired x-value, a mutable xpoints value if you will
        xmutable    = ma.copy(xpoints)
        #~ Set some to be compressed
        x_safe      = ma.compressed(xmutable)
        y_safe      = ma.compressed(ymasked)
        # Multiplier is used for smoothing. If there is no smoothing
        # the multiplier is 1. Otherwise it will be set as the number
        # of linspace divisions divided by the normal number of points.
        #~ print ymasked.filled(y_safe.mean())
        #~ print ymasked.filled( ymasked.mean() )
        #~ sys.exit(1)
        
        plot_color = color.next()
        
        multiplier  = lambda x: x
        if (not smoothing_is_wanted or smoothing_is_wanted is "Both") and options.graph:
            add_to_graph( contiguous , xmutable , yset , plot_color , multiplier , multiplier , title , unit)
        if smoothing_is_wanted:
            
            from scipy.interpolate import UnivariateSpline as spline
            
            divisions   = float(60)
            points      = ypoints[0].size
            ratio       = divisions / points
            if divisions%points == 0:
                step = ratio
            else:
                divisions   = round(ratio) * points
                step        = round(ratio)
            # Set our xmutable mask to the same as the filtered None
            xmutable.mask   = ymask
            # Create smoothing function based off of x and y points
            splinef         = spline(x_safe,y_safe)
            # Create set of points that is more/less granular than x
            # to allow for smooth curves
            xmutable        = np.linspace(start,end,divisions)
            #~ print 'X',xmutable
            #~ print 'Y', yset
            yset            = splinef(xmutable)
            start_multiplier        = lambda  x: x * step
            stop_multiplier         = lambda x: (x - 1) * step
            add_to_graph( contiguous , xmutable , yset , plot_color , start_multiplier ,stop_multiplier , title , unit)
        
        if extrapolate:
            
            import forecast
            
            #~ print method
            #~ print x_safe
            #~ sys.exit(1)
            
            if method == 'lstsq':
                yextrap = forecast.linear_regression(x_safe,y_safe,xlargest)
            elif method == 'dpf':
                yextrap = forecast.double_polyfit(x_safe,y_safe,xlargest)
            elif method == 'tpf':
                yextrap = forecast.triple_polyfit(x_safe,y_safe,xlargest)
            elif method == 'hw':
                #~ print xextrap.size
                yfilled = ymasked.filled(y_safe.mean())
                yextrap = forecast.holtwinters(yfilled,cycle,extrapolate)
                #~ print xextrap.size
                #~ print yextrap.size
                xlargest = xextrap
            if options.graph:
                add_to_graph( [ slice(0,xlargest.size) ] , xlargest , yextrap , plot_color , title = title + ' Extrapolated' , trans = .05)
            if options.xml:
                metrics.append({ 'title' : title , 'units' : unit })
                temp = []
                for i,date in enumerate(dates):
                    yindex  = utils.find_nearest( xextrap , matplotlib.dates.epoch2num(float(date)) )
                    value   = yextrap[yindex]
                    temp.append(value)
                values.append(temp)
    
    if options.graph:
        ax.plot_date(xlargest,np.zeros(xlargest.size),visible=False)
        fig.autofmt_xdate()
        plt.axis('Tight')
        plt.title(xmld.get('service','<service>') + ' on ' + xmld.get('host','<host>'))
        box = ax.get_position()
        if xmld.get('units',False):
            if xmld['units'][1:] == xmld['units'][:-1]:
                plt.ylabel(xmld['units'][0])
        ax.set_position([box.x0, box.y0 + box.height * 0.2, box.width, box.height * 0.8])
        ax.legend(loc='upper center', bbox_to_anchor=(.5,-.3),ncol=5)
        plt.grid()
        if options.dumpgraph:
            plt.savefig(options.dumpgraph)
        else:
            plt.savefig(sys.stdout)
    
    if options.xml:
        XML = utils.getXML( values , metrics , dates )
        print XML

def parseArgs():
    '''Wrapper to parse args and make it pretty.'''
    
    def verifyArgs( options ):
        '''Used to verify arguments passed our up to snuff.'''
        
        if options.rrdfile == "":
            print "\nMust give RRD file. Cannot continue without.\n"
            return False
        if not options.graph and not options.xml and not options.check:
            print "\nMust specify mode.\n"
            return False
        try: 
            rrdtool.fetch( options.rrdfile , 'AVERAGE' , '-s' , '-1d' )
        except:
            print "\nRRD file not readable.\n"
            return False
        return True
    
    from optparse import OptionParser, OptionGroup
    
    usage = sys.argv[0] + ' -f /path/to/file.rrd -[g/x/c] OPTIONS'
    
    parser = OptionParser(usage)
    #~ Make required option group
    group  = OptionGroup( parser , "Required Options", "The script will bail immediately if these are not specified.")
    group.add_option(  "-f","--rrdfile"
                    ,   help="RRD file to bring in to extrapolate data from."
                    ,   default=""
                    ,   type="str" )
    parser.add_option_group(group)
    #~ Make mode option group
    group  = OptionGroup( parser , "Mode", "Determines the mode of the script.")
    group.add_option(   "-g","--graph"
                    ,   help="Sets mode to graph. Script will return a graph."
                    ,   default=False
                    ,   action="store_true" )
    group.add_option(   "-x","--xml"
                    ,   help="Sets mode to return XML for data."
                    ,   default=False
                    ,   action="store_true" )
    group.add_option(   "-c","--check"
                    ,   help="Run as Nagios check, returning status and string."
                    ,   default=False
                    ,   action="store_true" )
    parser.add_option_group(group)
    #~ Make extra option group
    group = OptionGroup( parser, "Others" , "Miscellaneous options." )
    group.add_option(   "-r","--range"
                    ,   help="How far back the rrdtool will pull data from. Use only if using graph. [DEFAULT -1w]"
                    ,   default="-1w"
                    ,   type="str" )
    group.add_option(   "-s","--size"
                    ,   help="Size in inches for graph. [DEFAULT 10x3]"
                    ,   default='10x3' )
    group.add_option(   "-m","--smooth"
                    ,   help="Smooth the graph. Boolean flag. [DEFAULT: not present]"
                    ,   default=False
                    ,   action="store_true" )
    group.add_option(   "-t","--threshold"
                    ,   help="Set threshold to be reached. Used for extrapolation. [DEFAULT: 0 (None)]"
                    ,   default=0
                    ,   type="float" )
    group.add_option(   "-e","--extrapolate"
                    ,   help="Extrapolate out this many periods. [DEFAULT: 0 (None)]"
                    ,   default=0 
                    ,   type="int" )
    group.add_option(   "-i","--index"
                    ,   help="Pick specific index from the RRD file. [ACCEPTS integer]"
                    ,   default="-1" )
    group.add_option(   "-p","--period"
                    ,   help="Length of period. Usually this value would be something like one week or one month or one year. It is your best guess with the data you have as to what the cycle of the data is. If you are unsure then just leave this blank. [Options: n,w , n,d , n,m , n,y where n is some integer]"
                    ,   default="1,w" )
    group.add_option(   "-z","--method"
                    ,   help='Type of extrapolation used. Currently supported: Holt-Winters[hw], Least Squares[lstsq], Quadratic Fit [dpf], Cubic Fit [tpf], Levenburg-Marquardt[lm]. [DEFAULT: lstsq]'
                    ,   default='lstsq' )
    group.add_option(   "-d","--dates"
                    ,   help='Comma delimited list of dates you want checked. Taken in as time since epoch.'
                    ,   default='')
    group.add_option(   "--dumpgraph"
                    ,   help="Makes the graph PNGs print to /var/www/html/graph1.png"
                    ,   default=False )
    parser.add_option_group(group)
    (options, args) = parser.parse_args()
    if verifyArgs(options):
        return options
    else:
        parser.print_help()
        sys.exit(1)
        
def getRRDinfo( rrdfile , range , index = -1):
    '''
    @param rrdfile - location of rrdfile
    @param range   - range of dates to get
    @param index   - which index to pick from of the information in the RRD
    @return rrd_info dictionary
    ''' 
    try: 
        raw_rrd_info = rrdtool.fetch( rrdfile , 'AVERAGE' , '-s' , range )
    except:
        print "Error trying command raw_rrd_info = rrdtool.fetch( rrdfile , 'AVERAGE' , '-s' , $range )"
        sys.exit(1)
    #~ print ma.asarray(raw_rrd_info[2])[:,:]
    rrd_info = {    'start'     :   raw_rrd_info[0][0]
                ,   'end'       :   raw_rrd_info[0][1]
                ,   'res'       :   raw_rrd_info[0][2] }
    if index >= 0:
        try:
            rrd_info['titles']  = [ raw_rrd_info[1][index] ]
            rrd_info['ypoints'] = [ ma.asarray(raw_rrd_info[2])[:,index] ]
        except:
            print "Array index does not exist in RRD file."
            sys.exit(1)
    else:
        rrd_info['titles']  = raw_rrd_info[1]
        rrd_info['ypoints'] = []
        temp  = ma.asarray(raw_rrd_info[2])
        shape = temp.shape[1]
        for item in ma.arange(shape):
            rrd_info['ypoints'].append(temp[:,item])
    
    return rrd_info

def main():
    
    options     = parseArgs()
    index = int(options.index)
    rrd_info    = getRRDinfo( options.rrdfile , options.range , index )
    from utils import getRRDxml
    temp_xml    = options.rrdfile.replace('.rrd','.xml')
    rrd_xml     = getRRDxml( temp_xml )
    info( rrd_info , options , rrd_xml )
    
main()
