#!/usr/bin/env python

import numpy as np
import numpy.ma as ma
import sys

def holtwinters(ypoints,cycle_length,extrapolate_cycles,alpha=.11, beta=.001, gamma=.002,debug=False):
    '''This assumes that all ypoints passed have been PRE-SCREENED
    and are numpy or numpy.ma arrays.'''
    ylen = ypoints.size
    if not ylen % cycle_length == 0:
        print "ypoints must be a multiple of cycle_length."
        print "Truncating ypoints."
        reversed    = ypoints[::-1]
        cycles      = ylen / cycle_length
        reversed    = np.resize(reversed, cycles * cycle_length)
        ypoints     = reversed[::-1]
        ylen = ypoints.size
    #~ print ypoints.size
    fc = float(cycle_length)
    c  = cycle_length
    #~ Get the average of the second cycle of time series data.
    ybar2 = np.mean(ypoints[c:(2*c)])
    #~ Get the average of the first cycle of of time series data.
    ybar1 = np.mean(ypoints[:c])
    #~ Calcute initial b value from averages
    b_not = (ybar2 - ybar1) / fc
    if debug: print "b_not: ",b_not
    #~ Create a set of t points to go along with the ypoints
    tset = np.arange(1,c+1)
    tbar = np.mean(tset)
    #~ Use tbar to get initial alpha value
    a_not = ybar1 - b_not * tbar
    if debug: print "a_not: ",a_not
    #~ Construct Trend array
    #~ TODO: Find a more numpy approach to this problem
    I = np.asarray([ ypoints[i] / ( a_not + (i+1) * b_not ) for i in range(0,ylen)])
    if debug: print "Initial indices: ", I
    #~ Create empty array for season data
    S = (I[:c] + I[c:(2*c)]) / 2.0
    S = np.resize(S,ylen+(extrapolate_cycles*c))
    tS = c / np.sum(S[:c])
    S = S * tS
    if debug: print 'S:',S
    #~ Enter... the dragon
    F = np.zeros(ylen+c)
    E = np.zeros(c*extrapolate_cycles)
    A_t = a_not
    B_t = b_not
    for i in range(ylen):
        A_tm1   = A_t
        B_tm1   = B_t
        A_t     = alpha * ypoints[i] / S[i] + ( 1.0 - alpha ) * ( A_tm1 + B_tm1 )
        B_t     = beta  * ( A_t - A_tm1 ) + ( 1 - beta ) * B_tm1
        S[i+c]  = gamma * ypoints[i] / A_t + ( 1.0 - gamma ) * S[i]
        F[i]    = ( a_not + b_not * (i+1)) * S[i]
        #~ print "i=", i+1, "y=", y[i], "S=", S[i], "Atm1=", A_tm1, "Btm1=",B_tm1, "At=", A_t, "Bt=", B_t, "S[i+c]=", S[i+c], "F=", F[i]
    for m in np.arange(extrapolate_cycles*c):
        E[m] = ( A_t + B_t * (m+1)) * S[ylen + m]
        #~ print "Forecast: ", ( A_t + B_t * (m+1)) * S[ylen + m]
    E = ma.masked_less(E,0)
    return E.filled(0)

def linear_regression(xpoints,ypoints,xextrap):
    if not xpoints.size == ypoints.size:
        #~ print 'Dimension mismatch, attempting to guess.'
        xpoints = np.resize(xpoints,(1,ypoints.size))
        if not xpoints.size == ypoints.size:
            print 'Cannot match. Exiting.'
            return np.zeros(xpoints.size)
    A       = np.vstack([xpoints, np.ones(xpoints.size)]).T
    m, c    = np.linalg.lstsq(A,ypoints)[0]
    func    = np.vectorize(lambda x: m*x + c)
    yextrap = func(xextrap)
    return yextrap
    
def double_polyfit(xpoints,ypoints,xextrap):
    import warnings
    warnings.simplefilter('ignore', np.RankWarning)
    if not xpoints.size == ypoints.size:
        #~ print 'Dimension mismatch, attempting to guess.'
        xpoints = np.resize(xpoints,ypoints.size)
        if not xpoints.size == ypoints.size:
            print 'Cannot match. Exiting.'
            return np.zeros(xpoints.size)
    coefficients    = np.polyfit( xpoints , ypoints , 2 )
    coeff_func      = np.poly1d( coefficients )
    return coeff_func( xextrap )
    
def triple_polyfit(xpoints,ypoints,xextrap):
    import warnings
    warnings.simplefilter('ignore', np.RankWarning)
    if not xpoints.size == ypoints.size:
        #~ print 'Dimension mismatch, attempting to guess.'
        xpoints = np.resize(xpoints,ypoints.size)
        if not xpoints.size == ypoints.size:
            print 'Cannot match. Exiting.'
            return np.zeros(xpoints.size)
    coefficients    = np.polyfit( xpoints , ypoints , 3 )
    coeff_func      = np.poly1d( coefficients )
    return coeff_func( xextrap )
#~ y = np.array([146, 96, 59, 133, 192, 127, 79, 186, 272, 155, 98, 219])

#~ print holtwinters(y,4,52,.2,.1,.05)



    
