#!/usr/bin/env python

import matplotlib.dates as mdates
import matplotlib, sys, os, time
os.environ['TZ'] = 'America/Chicago'
time.tzset()
import numpy as np
from xml.etree.ElementTree import ElementTree
import scipy.stats,scipy.optimize
from scipy.interpolate import InterpolatedUnivariateSpline as spline
from random import choice
matplotlib.use('Agg')
matplotlib.rc('xtick',labelsize = 6)
matplotlib.rc('ytick',labelsize = 6)
matplotlib.rc('legend',fontsize = 6)
import matplotlib.pyplot as plt
import rrdtool


class cpGraph(object):
    
    colors      = [ 'green' , 'red' , 'blue' , 'gray' ]
    xlabel      = "time"
    ylabel      = []
    title       = ""
    ticks       = 8
    background  = "white"
    fill        = .1
    size        = ( 10 , 3 )
    smooth      = False
    extrap_res  = 1800
    extrapolate = 0
    titles      = []
    crit        = []
    warn        = []
    
    def __init__(self , rrdfile , begin , end = False , index = -1):
        self.fetch_data(rrdfile,begin,end)
        self.resolution     = self.working_points[0][2]
        self.start          = self.working_points[0][0]
        self.end            = self.working_points[0][1]
        self.xml            = self.get_xml(rrdfile)
        self.format_points(index)
        
    def fetch_data(self , filename , begin , end ):
        rrd_args = []
        rrd_args.append( filename + '.rrd' )
        rrd_args.append( 'AVERAGE' )
        rrd_args.append( '--start' )
        rrd_args.append( begin )
        if end:
                rrd_args.append( '--end' )
                rrd_args.append( end )
        rrd_data = rrdtool.fetch( *rrd_args )
        self.working_points = rrd_data
        
    def format_points(self , index):
        # Determine which indices we want
        if index < 0:
            index_list = range(len(self.working_points[1]))
        else:
            index_list = [ index ]
        # Initialize empty dataset
        raw_point_list      = self.working_points[2]
        length              = len(raw_point_list)
        self.datasets       = []
        for i in index_list:
            self.titles.append(self.working_points[1][i])
            y_coord = map(lambda x: x[i],raw_point_list)
            x_coord = range(self.start,self.end,self.resolution)
            # Now we remove the None points
            for j in range(y_coord.count(None)):
                bad_index = y_coord.index(None)
                y_coord[bad_index] = 0
            self.datasets.append([x_coord,y_coord])
    
    def smooth_points(self):
        temp_points = []
        x_extrap    = range(self.start,self.end,5)
        for points in self.datasets:
            x,y = self.remove_value(points,0)
            div     = len(x) / 65
            x = x[::div]
            y = y[::div]
            curve       = spline(x,y)
            y_extrap    = curve(x_extrap)
            y_extrap = self.min_value(0 , y_extrap) 
            temp_points.append([x_extrap,y_extrap])
        return temp_points
    
    def create_graph(self, interpolate = 0):
        fig = plt.figure(figsize = self.size)
        plt.xlabel(self.xlabel)
        plt.ylabel('/'.join(self.ylabel))
        if not self.title:
            plt.title('Unnamed Graph')
        else:
            plt.title( self.title )
        print self.datasets[0][0]
        if self.smooth:
            self.datasets = self.smooth_points()
        k = 0
        for i in self.datasets:
            print 'doing ' + str(k) + ' of datasets'
            x = map(matplotlib.dates.epoch2num,i[0])
            if not self.extrapolate:
                plt.plot_date(x,i[1],visible=False)
            print 'doing lin reg'
            print 'plotting extrap'
            plt.plot(x,i[1],color = self.colors[k],label = self.titles[k])
            plt.fill_between(x,i[1],color = self.colors[k],alpha = self.fill)
            k = k + 1
        if self.extrapolate:
            relation, r = self.get_association(self.datasets[0])
            x, y = self.linear_regression( self.datasets[0] , self.extrapolate )
            x = map(matplotlib.dates.epoch2num,x)
            plt.plot_date(x,y,visible=False)
            plt.plot(x,y,'--', color = 'black', linewidth = 2 , label = 'Extrapolation\n' + relation + ':' + str(r))
            plt.fill_between(x,y,color = 'black',alpha = .12)
        fig.autofmt_xdate()
        plt.axis(ymin = 0)
        plt.legend(bbox_to_anchor=(.993, 1.04), loc=2, borderaxespad=1)
        plt.subplots_adjust(bottom=0.2)
        plt.grid()
        plt.draw()
        plt.savefig('/usr/local/nagiosxi/html/graph1.png')
        
    def make_residual_line(self, dataset):
        x, y    = dataset[0],dataset[1]
        func    = np.poly1d(np.polyfit(x,y,5))
        return func
    
    def min_value(self , x , lat):
        for i in range(len(lat)):
            if lat[i] < x:
                lat[i] = x
        return  lat
        
    def get_association(self, dataset):
        r = scipy.stats.pearsonr(dataset[0],dataset[1])[0]
        if r >= .7:
            relation = 'Strong Pos'
        elif r >= .3:
            relation = 'Weak Pos'
        elif r >= -.3:
            relation = 'Not Linear'
        elif r >= -.7:
            relation = 'Weak Neg'
        else:
            relation = 'Strong Neg'
        return relation , round(r,2)
        
    def linear_regression(self , dataset , length = 0, return_func = False):
        x_coord, y_coord = self.remove_value(dataset,0)
        A       = np.vstack([x_coord, np.ones(len(x_coord))]).T
        m, c    = np.linalg.lstsq(A,y_coord)[0]
        func = lambda x: (m * x) + c
        if return_func:
            return func
        for i in range(1,length+1):
            x_coord.append(x_coord[-1]+(i*self.extrap_res))
        y_coord = map(func , x_coord)
        return x_coord , y_coord
        
    def remove_value(self , dataset , subject , replacement = None):
        x,y = dataset[0],dataset[1]
        i = 0
        for i in range(len(y)):
            try:
                if y[i] == subject:
                    if replacement == None:
                        del y[i]
                        del x[i]
                    else:
                        y[i] = replacement
            except:
                pass
        return x , y

    def get_xml(self , rrdfile ):
        tree = ElementTree()
        info = tree.parse( rrdfile + '.xml' )
        self.title  = str(info.find('NAGIOS_AUTH_SERVICEDESC').text) + ' on ' + str(info.find('NAGIOS_HOSTNAME').text)
        DATA = info.findall('DATASOURCE')
        for items in DATA:
            temp = items.find('UNIT').text
            self.ylabel.append(temp)
            self.titles.append(items.find('LABEL').text)
            self.warn.append(items.find('WARN').text)
            self.crit.append(items.find('CRIT').text)
            
    def finalize_graph(self):
        plt.draw()
        plt.savefig('/usr/local/nagiosxi/html/graph1.png')
        
        
        
#~ x = cpGraph(rrdfile = '/usr/local/nagios/share/perfdata/localhost/Current_Load', begin = '-6d' , end = False , index = -1)
#~ x.title = 'CPU Load (localhost)'
#~ x.smooth = True
#~ x.extrapolate = 12
#~ x.create_graph()

x = cpGraph(rrdfile = '/usr/local/nagios/share/perfdata/localhost/PING', begin = '-10h' , end = '-8h' , index = 0)
x.smooth = True
x.extrapolate = 0
x.create_graph()
x.finalize_graph()
#~ 
#~ tree = ElementTree()
#~ info = tree.parse ( '/usr/local/nagios/share/perfdata/localhost/Current_Load.xml' )
