<?php
// CAPACITY PLANNING REPORT
//
// Copyright (c) 211 Nagios Enterprises, LLC.  All rights reserved.
//  
// $Id: capacityplanning.php 1050 2012-02-28 22:31:02Z egalstad $

require_once(dirname(__FILE__).'/../includes/common.inc.php');

// initialization stuff
pre_init();

// start session
init_session();

// grab GET or POST variables 
grab_request_vars();

// check prereqs
check_prereqs();

// check authentication
check_authentication(false);


route_request();

function route_request(){
	global $request;
	
	/*
	if(is_admin()==false){
		echo "You are not authorized to view this report";
		exit();
		}
	*/
	
	$mode=grab_request_var("mode","");
	switch($mode){
		case "getdata":
			get_report_data();
			break;
		default:
			display_report();
			break;
		}
	}


	
///////////////////////////////////////////////////////////////////
// REPORT GENERATION FUCNTIONS
///////////////////////////////////////////////////////////////////

function get_report_data(){

	$host=grab_request_var("host","");
	$service=grab_request_var("service","_HOST_");
	
	$type=grab_request_var("type","xml"); // XML or Graph?
	$range=grab_request_var("range","-8w"); // pull last 2 weeks from RRD file
	$period=grab_request_var("period","1,w"); // 1 week period
	$extrapolate=grab_request_var("extrapolate","10"); // extrapolate 1 week in the future
	$method=grab_request_var("method",""); // hw=Holt-Winters
	$dates=grab_request_var("dates",""); // dates
	
	// rrd file to use
	$f=pnp_get_perfdata_file($host,$service);
	
	// set environment variable
	putenv("MPLCONFIGDIR=/tmp");
	
	$cmd="/usr/local/nagiosxi/html/includes/components/cp/cpGraph.py -f ".escapeshellarg($f)."";
	if($type=="xml"){
		$cmd.=" -x";
		}
	else{
		$cmd.=" -g";
		$cmd.=" -r ".escapeshellarg($range)."";  // use range only if producing a graph
		//$cmd.=" -x 6x2"; // default=10x3
		}
	$cmd.=" -p ".escapeshellarg($period)."";
	$cmd.=" -e ".escapeshellarg($extrapolate)."";
	if($method!="")
		$cmd.=" -z ".escapeshellarg($method)."";
	if($dates!="")
		$cmd.=" -d ".escapeshellarg($dates)."";
	
	if($type=="xml"){
		//echo $cmd;
		//echo "<BR><BR>";
		}
	else{
		header('Content-type: image/png');
		}
	
	passthru($cmd);
	
	exit();
	}

// this function displays report in HTML
function display_report(){
	global $lstr;
	global $request;

	// makes sure user has appropriate license level
	licensed_feature_check();
	
	// get values passed in GET/POST request
	$page=grab_request_var("page",1);
	$records=grab_request_var("records",25);
	
	$host=grab_request_var("host","");
	$service=grab_request_var("service","");
	$timeperiod=grab_request_var("timeperiod","");
	$method=grab_request_var("method","");
	
	
	// REDIRECT USER TO REPORT FOR FIRST VALID HOST IF NONE SPECIFIED
	if($host==""){
		$args=array('brevity' => 1); 
		$oxml=get_xml_host_objects($args);
		if($oxml){
			foreach($oxml->host as $hostobject){
				$name=strval($hostobject->host_name);
				$have_cpdata=capacity_planning_data_exists($name);
				if($have_cpdata==true){
					$newurl="?host=".urlencode($name);
					header("Location: ".$newurl);
					}
				}
			}
		}

	// start the HTML page
	do_page_start(array("page_title"=>"Capacity Planning"),true);
	
?>
	<h1>Capacity Planning</h1>

<?php
	if($service!=""){
?>
	<div class="servicestatusdetailheader">
	<div class="serviceimage">
	<!--image-->
	<?php show_object_icon($host,$service,true);?>
	</div>
	<div class="servicetitle">
	<div class="servicename"><a href="<?php echo get_service_status_detail_link($host,$service);?>"><?php echo htmlentities($service);?></a></div>
	<div class="hostname"><a href="<?php echo get_host_status_detail_link($host);?>"><?php echo htmlentities($host);?></a></div>
	</div>
	</div>
	<br clear="all">

<?php
		}
	else if($host!=""){
?>
	<div class="hoststatusdetailheader">
	<div class="hostimage">
	<!--image-->
	<?php show_object_icon($host,"",true);?>
	</div>
	<div class="hosttitle">
	<div class="hostname"><a href="<?php echo get_host_status_detail_link($host);?>"><?php echo htmlentities($host);?></a></div>
	</div>
	</div>
	<br clear="all">
<?php
		}
?>

	
<?php
	
?>
	<form method="get" action="<?php echo htmlentities($_SERVER["REQUEST_URI"]);?>">
	
	<div class="reportexportlinks">
	<?php echo get_add_myreport_html("Event Log",$_SERVER["REQUEST_URI"],array());?>
<?php
	$url="?1";
	foreach($request as $var => $val)
		$url.="&".urlencode($var)."=".urlencode($val);
?>
	<!--
	<a href="<?php echo $url;?>&mode=pdf" alt="Download As PDF" title="Download As PDF"><img src="<?php echo theme_image("pdf.png");?>"></a>
	//-->
	</div>
	
	
<?php
	$auto_start_date=date('m/d/Y',strtotime('yesterday'));
	$auto_end_date=date('m/d/Y');
?>
	
	<script type="text/javascript">
	$(document).ready(function(){
		$('#startdateBox').click(function(){
			$('#reportperiodDropdown').val('custom');
			if($('#startdateBox').val()=='' && $('#enddateBox').val()==''){
				$('#startdateBox').val('<?php echo $auto_start_date;?>');
				$('#enddateBox').val('<?php echo $auto_end_date;?>');
				}
			});
		$('#enddateBox').click(function(){
			$('#reportperiodDropdown').val('custom');
			if($('#startdateBox').val()=='' && $('#enddateBox').val()==''){
				$('#startdateBox').val('<?php echo $auto_start_date;?>');
				$('#enddateBox').val('<?php echo $auto_end_date;?>');
				}
			});
		
		});
	</script>

	<div class="reporttimepicker">
	Period&nbsp;
	<select name="timeperiod">
<?php
	$tp=get_capacity_planning_timeperiod_options();
	foreach($tp as $shortname => $arr){
		echo "<option value='".$shortname."' ".is_selected($shortname,$timeperiod).">".$arr["description"]."</option>";
		}
?>
	</select>
	
	Method&nbsp;
	<select name="method">
	<option value="">Auto</option>
<?php
	$pm=get_capacity_planning_methods();
	foreach($pm as $shortname => $longname){
		echo "<option value='".$shortname."' ".is_selected($shortname,$method).">".$longname."</option>";
		}
?>
	</select>	
		
	Host&nbsp;

	<select name="host" id="hostList">
	<option value=""></option>
<?php
	$args=array('brevity' => 1); 
	$oxml=get_xml_host_objects($args);
	if($oxml){
		foreach($oxml->host as $hostobject){
			$name=strval($hostobject->host_name);
			$have_cpdata=capacity_planning_data_exists($name);
			echo "<option value='".$name."' ".is_selected($host,$name).">$name";
			if($have_cpdata==false)
				echo "  *";
			echo "</option>\n";
			}
		}
?>
	</select>	
	
<?php
	if($host!=""){
?>
<!-- DO NOT USE -->
<!--
	Service&nbsp;

	<select name="service" id="serviceList">
	<option value=""></option>
<?php
	$args=array(
		'brevity' => 1,
		'host_name' => $host,
		); 
	$oxml=get_xml_service_objects($args);
	if($oxml){
		foreach($oxml->service as $svcobject){
			$name=strval($svcobject->service_description);
			$have_cpdata=capacity_planning_data_exists($host,$name);
			echo "<option value='".$name."' ".is_selected($service,$name).">$name";
			if($have_cpdata==false)
				echo "  *";
			echo "</option>\n";
			}
		}
?>
	</select>	
	//-->
<?php
		}
?>

	<input type='submit' class='reporttimesubmitbutton' name='reporttimesubmitbutton' value='Go'>
	</div>

	
	

<?php
	// report on specific service
	if($host!="" && $service!=""){
		if(capacity_planning_data_exists($host,$service)==false){
			echo "<p><b>ERROR:</b> Capacity planning data is not available for the selected service.</p>";
			echo "<a href='?host=".urlencode($host)."'>Show all host services</a>";
			}
		else{
			echo "<p>DEBUG: Capacity planning data exists...</p>";
			
			
			
			$imgurl="?mode=getdata&type=graph&host=".urlencode($host)."&service=".urlencode($service);
			$imgurl.=add_capacity_planning_data_url_opts();
			echo "<div style='margin: 0 10px 10px 0; float: left;'>";
			echo "DEBUG: URL=<b>$imgurl</b><BR>";
			echo "<img src='".$imgurl."' border='1' alt='Graph' title='Graph'></div>";
			
			echo "<div style='margin: 0 10px 10px 0; float: left; border: 1px solid gray;'>HTML TABLE DATA GOES HERE...</div>";

			echo "<br clear='all'>";

			echo "<div style='margin-top: 10px;'><a href='?host=".urlencode($host)."'>Show all data for ".htmlentities($host)."</a></div>";
			
			}
		}
	// report on specific host
	else if($host!=""){
		if(capacity_planning_data_exists($host)==false){
			echo "<p><b>ERROR:</b> Capacity planning data is not available for the selected host. Select another host from the dropdown box to continue.</p>";
			//echo "<a href='?host='>Show all hosts</a>";
			}
		else{
			echo "<p>DEBUG: Capacity planning data exists...</p>";
			
			$cpsvcs=get_capacity_planning_service_names($host);
			
			$imgurl="?mode=getdata&type=graph&host=".urlencode($host);
			$imgurl.=add_capacity_planning_data_url_opts();
			echo "<div style='margin: 0 10px 10px 0; float: left;'>";
			echo "DEBUG: URL=<b>$imgurl</b><BR>";
			echo "<img src='".$imgurl."' border='1' alt='Graph' title='Graph'></div>";

			echo "<div style='margin: 0 10px 10px 0; float: left; border: 1px solid gray;'>HTML TABLE DATA GOES HERE...</div>";

			echo "<br clear='all'>";

			//echo "<div><a href='?host='>Show all hosts</a></div>";

			echo "<p><strong>Services</strong>:</p>";
			
			foreach($cpsvcs as $sn){
				$have_cpdata=capacity_planning_data_exists($host,$sn);
				if($have_cpdata==true)
					echo "<a href='?host=".urlencode($host)."&service=".urlencode($sn)."'>";
				echo htmlentities($sn);
				if($have_cpdata==true)
					echo "</a>";
				//if($have_cpdata==true)
					//echo " DEBUG: <b>CAPACITY PLANNING DATA EXISTS</b>";
				if($have_cpdata==false)
					echo " *";
				echo "<br>";
				}

			}
		}
?>



<?php		
/*
	echo "<p>VALID HOSTS:</p>";
	
	$cphostnames=get_capacity_planning_host_names();
	
	foreach($cphostnames as $hn){
		$have_cpdata=capacity_planning_data_exists($hn);
		echo "<a href='?host=".urlencode($hn)."'>".htmlentities($hn)."</a>";
		if($have_cpdata==true)
			echo " <b>EXISTS</b>";
		echo "<br>";
		}
*/

	
	// closes the HTML page
	do_page_end(true);
	}
	
	
function capacity_planning_data_exists($hostname="",$servicename="_HOST_"){

	return perfdata_chart_exists($hostname,$servicename);
	}
	
function get_capacity_planning_host_names(){

	$hosts=array();

	$args=array('brevity' => 1); 
	$oxml=get_xml_host_objects($args);
	if($oxml){
		foreach($oxml->host as $hostobject){
			$name=strval($hostobject->host_name);
			$hosts[]=$name;
			}
		}
		
	return $hosts;
	}
	
function get_capacity_planning_service_names($hostname){

	$services=array();

	$args=array(
		'brevity' => 1,
		'host_name' => $hostname,
		); 
	$oxml=get_xml_service_objects($args);
	if($oxml){
		foreach($oxml->service as $svcobject){
			$name=strval($svcobject->service_description);
			$services[]=$name;
			}
		}
		
	return $services;
	}
	
function get_capacity_planning_timeperiod_options(){

	$opts=array(
		"1month" => array(
			"description" => "1 Month",
			"range" => "-1m",
			"period" => "1,d",
			"extrapolate" => "31",
			"dates" => strtotime("+1 month"),
			),
		"3months" => array(
			"description" => "3 Months",
			"range" => "-3m",
			"period" => "1,m",
			"extrapolate" => "3",
			"dates" => strtotime("+3 months"),
			),
		"6months" => array(
			"description" => "6 Months",
			"range" => "-6m",
			"period" => "1,m",
			"extrapolate" => "6",
			"dates" => strtotime("+6 months"),
			),
		"1year" => array(
			"description" => "1 Year",
			"range" => "-12m",
			"period" => "1,m",
			"extrapolate" => "12",
			"dates" => strtotime("+12 months"),
			),
		);
	
	return $opts;
	}
	
function get_capacity_planning_methods(){
	
	$opts=array(
		"hw" => "Holt-Winters",
		"lstsq" => "Least Squares",
		"dpf" => "Quadratic Fit",
		"tpf" => "Cubic Fit",
		//"lm" => "Levenburg-Marquardt",
		);
	return $opts;
	}

function add_capacity_planning_data_url_opts(){
	$urlopts="";
	
	$method=grab_request_var("method","");
	$timeperiod=grab_request_var("timeperiod",""); // this is a different period than passed to data backend...
	
	
	// find pre-determined options for this timeperiod
	$topts=get_capacity_planning_timeperiod_options();
	$popts=grab_array_var($topts,$timeperiod,array());
	
	$range=grab_array_var($popts,"range","");
	$period=grab_array_var($popts,"period","");
	$extrapolate=grab_array_var($popts,"extrapolate","");
	$dates=grab_array_var($popts,"dates","");
	
	// add pre-determined options to url
	if($range!="")
		$urlopts.="&range=".urlencode($range);
	if($period!="")
		$urlopts.="&period=".urlencode($period);
	if($extrapolate!="")
		$urlopts.="&extrapolate=".urlencode($extrapolate);
	if($dates!="")
		$urlopts.="&dates=".urlencode($dates);

	// user can specify algorithm/method
	$urlopts.="&method=".urlencode($method);

	
	return $urlopts;
	}

?>

